# Copyright 2015 Cisco Systems, Inc.
# All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Test app"""

import unittest
from yabgp.api.app import app


class FlaskrTestCase(unittest.TestCase):

    def setUp(self):
        # prepare some peers
        self.app = app.test_client()

    def tearDown(self):
        """"""
        pass

    def test_show_bgp_summary(self):
        result = self.app.get('/')
        self.assertTrue(result)

if __name__ == '__main__':
    unittest.main()
